#!/bin/bash
dll=$1
directory=$2

# Some sanity checking.
if [ ! -f "./$dll" ] ; then
    echo "file with name of $dll not found in the current directory"
    exit 1
fi

if [ ! -d "./$directory" ] ; then
    echo "subdirectory with name of $directory not found in the current directory"
    exit 1
fi

echo "
Properly replace cygwin1.dll by $dll in $directory
(using cygwin_staging as a disposable staging area)"

ANSWER=
while [ "$ANSWER" != "yes" -a "$ANSWER" != "no" ] ; do
    echo -n "Continue (yes/no)? "
    read ANSWER
done
if [ "$ANSWER" = "no" ] ; then
    echo "Immediate exit specified!"
    exit
fi

rm -rf cygwin_staging
mkdir cygwin_staging
cd cygwin_staging

original_tarball=$(find ../$directory -name "cygwin*tar.bz2")
tarball_name=$(basename $original_tarball)
# MSYS tar has difficulty with symlinks because of the well-known lack
# of a dangling symlink implementation on MSYS so work around the
# issue by unpacking the tarball twice.  The first unpack succeeds
# with all files, but fails for symlinks that appear before the file
# they point to in the tarball (which on Linux would create a dangling
# symlink that is filled in later with no issues). The second unpack
# works for all symlinks because the files they point to are already
# on disk.  (Such a double unpacking should be harmless on systems
# that handle dangling symlinks with no issues.)  The errors for
# symlinks (if any) associated with the first unpack are captured into
# a file to avoid outputting confusing error messages.
tar jxf $original_tarball 2> initial_tar_error
tar jxf $original_tarball
cp -a ../$dll usr/bin/cygwin1.dll

tar jcf $tarball_name etc usr

original_size=$(wc --bytes $original_tarball | cut --delimiter=" " --fields=1)
original_md5sum=$(md5sum $original_tarball |cut --delimiter=" " --fields=1)
echo "original_size = $original_size"
echo "original_md5sum = $original_md5sum"

modified_size=$(wc --bytes $tarball_name | cut --delimiter=" " --fields=1)
modified_md5sum=$(md5sum $tarball_name |cut --delimiter=" " --fields=1)
echo "modified_size = $modified_size"
echo "modified_md5sum = $modified_md5sum"

original_setup_ini=$(find ../$directory -name "setup.ini")
#echo "original_setup_ini = ${original_setup_ini}"
sed \
-e"/$original_size *$original_md5sum/s?$original_size?$modified_size?" \
-e"/$modified_size *$original_md5sum/s?$original_md5sum?$modified_md5sum?" \
$original_setup_ini > setup.ini

# Now overwrite $directory with updated information.
cp $tarball_name $original_tarball
cp setup.ini $original_setup_ini
